import 'dart:math';

import 'package:flutter/material.dart';

import 'package:calculus/config/default_game_settings.dart';
import 'package:calculus/models/settings/settings_game.dart';
import 'package:calculus/models/settings/settings_global.dart';
import 'package:calculus/utils/tools.dart';

class ParameterPainter extends CustomPainter {
  const ParameterPainter({
    required this.code,
    required this.value,
    required this.gameSettings,
    required this.globalSettings,
  });

  final String code;
  final String value;
  final GameSettings gameSettings;
  final GlobalSettings globalSettings;

  @override
  void paint(Canvas canvas, Size size) {
    // force square
    final double canvasSize = min(size.width, size.height);

    // content
    switch (code) {
      case DefaultGameSettings.parameterCodeLevel:
        paintLevelParameterItem(canvas, canvasSize);
        break;
      default:
        printlog('Unknown parameter: $code/$value');
        paintUnknownParameterItem(canvas, canvasSize);
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }

  // "unknown" parameter -> simple block with text
  void paintUnknownParameterItem(
    final Canvas canvas,
    final double size,
  ) {
    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    final textSpan = TextSpan(
      text: '$code\n$value',
      style: const TextStyle(
        color: Colors.black,
        fontSize: 18,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintLevelParameterItem(
    final Canvas canvas,
    final double size,
  ) {
    final List<dynamic> stars = [];

    switch (value) {
      case DefaultGameSettings.levelValueEasy:
        stars.add([0.5, 0.5]);
        break;
      case DefaultGameSettings.levelValueMedium:
        stars.add([0.3, 0.5]);
        stars.add([0.7, 0.5]);
        break;
      case DefaultGameSettings.levelValueHard:
        stars.add([0.3, 0.3]);
        stars.add([0.7, 0.3]);
        stars.add([0.5, 0.7]);
        break;
      case DefaultGameSettings.levelValueNightmare:
        stars.add([0.3, 0.3]);
        stars.add([0.7, 0.3]);
        stars.add([0.3, 0.7]);
        stars.add([0.7, 0.7]);
        break;
      default:
        printlog('Wrong value for level parameter value: $value');
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3 / 100 * size;

    // Stars
    final textSpan = TextSpan(
      text: '⭐',
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 3,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();

    for (var center in stars) {
      textPainter.paint(
        canvas,
        Offset(
          size * center[0] - textPainter.width * 0.5,
          size * center[1] - textPainter.height * 0.5,
        ),
      );
    }
  }
}
